<?php

class Encodings
{
	function &getInstance( $language = 'uni', $order = 'auto' )
	{
		static $instance = NULL;

		if ( $instance === NULL ) {
			if ( ! extension_loaded( 'mbstring' ) ) {
				$instance = new Encodings();
			}
			else {
				$instance = new Multibyte_Encodings( $language, $order );
			}
		}
		return $instance;
	}
	function convert( $str )         { return $str; }
	function convertArgs( $args )    { return $args; }
	function convertArray( $args )   { return $args; }
	function convertEncoding( $str ) { return $str; }
	function detectEncoding()        { return 'UTF-8'; }
	function getCharset()            { return 'UTF-8'; }
	function getEncodings()          { return array(); }
	function getHtmlCharset()        { return ''; }
	function setEncoding()           { return 'None'; }
	function setEncodings()          { return FALSE; }
	function setOutputEncoding()     { ; }
}

class Multibyte_Encodings
{
	var $encodings;
	var $internalEncoding;
	var $fromEncoding;
	var $encodingOrder;

	function Multibyte_Encodings( $enc = 'uni', $order = 'auto' )
	{
		$this->encodingOrder    = $order;
		$this->encodings        = array();
		$this->fromEncoding     = '';
		$this->internalEncoding = 'UTF-8';

		if ( ! preg_match( '/^uni|Ja(?:panese)?|En(?:glish)?$/i', $enc ) ) {
			$enc = 'uni';
		}
		mb_language( $enc );
		mb_internal_encoding( $this->internalEncoding );
	}

	function convert( $str )
	{
		if ( ! $this->fromEncoding ) {
			$this->fromEncoding = $this->encodingOrder;
		}
		if ( $this->fromEncoding !== $this->internalEncoding ) {
			$str = mb_convert_encoding( $str, $this->internalEncoding, $this->fromEncoding );
		}
		return $str;
	}

	function convertArgs( $args )
	{
		$from_enc = '';
		if ( ini_get( 'mbstring.encoding_translation' ) ) {
			if ( ini_get( 'mbstring.http_input' ) !== 'pass' ) {
				$from_enc = ini_get( 'mbstring.internal_encoding' );
			}
			else if ( mb_http_output() !== 'pass' ) {
				$from_enc = mb_http_output();
			}
		}
		else if ( mb_http_output() !== 'pass' ) {
			$from_enc = mb_http_output();
		}
		if ( $from_enc ) {
			$this->fromEncoding = $from_enc;
			$args = $this->convertArray( $args );
			$this->fromEncoding = '';
		}
		return $args;
	}

	function convertArray( $args )
	{
		if ( is_array( $args ) ) {
			return array_map( array( &$this, 'convertArray' ), $args );
		}
		return mb_convert_encoding( $args, $this->internalEncoding, $this->fromEncoding );
	}

	function convertEncoding( $str, $to_enc, $from_enc = 'auto' )
	{
		return mb_convert_encoding( $str, $to_enc, $from_enc );
	}

	function detectEncoding( $str, $encoding = FALSE )
	{
		if ( ! $this->encodings ) {
			$this->setEncodings();
		}
		if ( ! $encoding || ! isset( $this->encodings[ $encoding ] ) ) {
			$encoding = mb_detect_encoding( $str, $this->encodingOrder );
		}
		$this->fromEncoding = $encoding;
		return $encoding;
	}

	function getCharset()
	{
		$enc = mb_http_output();
		if ( $enc === 'pass' ) {
			$enc = $this->internalEncoding;
		}
		switch ( strtoupper( $enc ) ) {
		case 'SJIS'       :
		case 'SJISJP-WIN' : return 'Shift_JIS';
		case 'EUC-JP'     :
		case 'EUCJP-WIN'  : return 'EUC-JP';
		case 'JIS'        : return 'ISO-2022-JP';
		default           : return $enc;
		}
	}

	function getEncodings()
	{
		if ( ! $this->encodings ) {
			$this->setEncodings();
		}
		return $this->encodings;
	}

	function getHtmlCharset( $str )
	{
		if ( preg_match( '/(?:charset|encoding)\s*=\s*[\'"]?([-\w]+)\\1?/i', $str, $m ) ) {
			switch ( strtoupper( $m[1] ) ) {
			case 'X-EUC-JP'     :
			case 'EUC-JP'       :
			case 'EUC_JP'       : return 'eucJP-win';
			case 'ISO-2022-JP'  : return 'JIS';
			case 'X-SJIS'       :
			case 'SHIFT_JIS'    : return 'SJIS-win';
			case 'UTF-8'        : return 'UTF-8';
			case 'ISO-8859-1'   : return 'ISO-8859-1';
			case 'WINDOWS-1250' : return 'Windows-1251';
			case 'WINDOWS-1252' : return 'Windows-1252';
			}
		}
		return '';
	}

	function setEncodings()
	{
		$this->encodings = array(
			''             => TRUE,
			'7bit'         => TRUE,
			'8bit'         => TRUE,
			'ASCII'        => TRUE,
			'BASE64'       => TRUE,
			'BIG-5'        => TRUE,
			'CP866'        => TRUE,
			'CP936'        => TRUE,
			'EUC-CN'       => TRUE,
			'EUC-JP'       => TRUE,
			'EUC-KR'       => TRUE,
			'EUC-TW'       => TRUE,
			'HZ'           => TRUE,
			'ISO-2022-JP'  => TRUE,
			'ISO-2022-KR'  => TRUE,
			'ISO-8859-1'   => TRUE,
			'ISO-8859-2'   => TRUE,
			'ISO-8859-3'   => TRUE,
			'ISO-8859-4'   => TRUE,
			'ISO-8859-5'   => TRUE,
			'ISO-8859-6'   => TRUE,
			'ISO-8859-7'   => TRUE,
			'ISO-8859-8'   => TRUE,
			'ISO-8859-9'   => TRUE,
			'ISO-8859-10'  => TRUE,
			'ISO-8859-13'  => TRUE,
			'ISO-8859-14'  => TRUE,
			'ISO-8859-15'  => TRUE,
			'JIS'          => TRUE,
			'KOI8-R'       => TRUE,
			'SJIS'         => TRUE,
			'SJIS-win'     => TRUE,
			'UCS-2'        => TRUE,
			'UCS-2BE'      => TRUE,
			'UCS-2LE'      => TRUE,
			'UCS-4'        => TRUE,
			'UCS-4BE'      => TRUE,
			'UCS-4LE'      => TRUE,
			'UHC'          => TRUE,
			'UTF-16'       => TRUE,
			'UTF-16BE'     => TRUE,
			'UTF-16LE'     => TRUE,
			'UTF-32'       => TRUE,
			'UTF-32BE'     => TRUE,
			'UTF-32LE'     => TRUE,
			'UTF-7'        => TRUE,
			'UTF-8'        => TRUE,
			'UTF7-IMAP'    => TRUE,
			'Windows-1251' => TRUE,
			'Windows-1252' => TRUE,
			'byte2be'      => TRUE,
			'byte2le'      => TRUE,
			'byte4be'      => TRUE,
			'byte4le'      => TRUE,
			'eucJP-win'    => TRUE,
		);
	}
	
	function setOutputEncoding( $enc )
	{
		if ( $enc !== $this->internalEncoding ) {
			if ( ! $this->encodings ) {
				$this->setEncodings();
			}
			if ( isset( $this->encodings[$enc] ) ) {
				mb_http_output( $enc );
				ob_start( 'mb_output_handler' );
			}
		}
		return TRUE;
	}
}

?>
