<?php

function addSlash( $str )
{
	return rtrim( $str, '/' ) . '/';
}

function diffTime( $st = NULL )
{
	static $ts = NULL;
	if ( $ts === NULL ) {
		$ts = $st ? $st : microtime();
		return TRUE;
	}
	list( $ts_usec, $ts_sec ) = explode( ' ', $ts );
	list( $te_usec, $te_sec ) = explode( ' ', microtime() );
	return ( (float)$te_usec - (float)$ts_usec ) + (float)( (int)$te_sec - (int)$ts_sec );
}

/*
 * see saveConfig
 */
function loadConfig( $file )
{
	// for PHP 4.2.4-dev : against Segmentation Fault
	if ( @filesize( $file ) === 0 ) {
		return array();
	}
	return @include( $file );
}

/**
 * rm -rf [dir]
 */
function remove_dir( $dir )
{
	$dir = rtrim( $dir, '/' );
	if ( is_dir( $dir ) ) {
		$dh = opendir( $dir );
		while ( FALSE !== ( $file = readdir( $dh ) ) ) {
			if ( $file === '.' || $file === '..' ) {
				continue;
			}
			remove_dir( $dir . '/' . $file );
		}
		rmdir( $dir );
		return;
	}
	unlink( $dir );
}

function saveArray( $filename, $array )
{
	// for var_export() bug. see http://bugs.php.net/bug.php?id=25758
	if ( version_compare( PHP_VERSION, '4.3.4', '<' ) ) {
		$arr = array();
		foreach ( $array as $key => $val ) {
			$key = str_replace( array( '\\', '\'' ), array( '\\\\', '\\\'' ), $key );
			$arr[$key] = $val;
		}
		$array = $arr;
	}
	// for var_export() in PHP 4.3.10/4.3.10RC1/4.3.10RC2 bug. see http://bugs.php.net/bug.php?id=31072
	if ( version_compare( PHP_VERSION, '4.3.10', '<=' )
		&& version_compare( PHP_VERSION, '4.3.9',  '>' )
		&& ! empty( $array[''] )
	) {
		$code = var_export( array( $md5 = md5( microtime() ) => $array[''] ) + $array, TRUE );
		$code = preg_replace( "/" . $md5 . "/", '', $code );
	}
	else {
		$code = var_export( $array, TRUE );
	}
	$code = '<?php return ' . $code . ' ?>';
	$result = file_put_contents( $filename, $code );
	return is_int( $result );
}

/*
 * see loadConfig
 */
function saveConfig( $file, $array )
{
	if ( ! is_writable( $file ) && ! @touch( $file ) ) {
		return FALSE;
	}
	$tmpfile = $file . '.tmp';
	if ( ! saveArray( $tmpfile, $array ) ) {
		return FALSE;
	}
	// direct rename() is not supported in Windows
	if ( substr( PHP_OS, 0, 3 ) === 'WIN' ) {
		@unlink( $file );
	}
	clearstatcache();

	return rename( $tmpfile, $file );
}

function sanitize( $arr )
{
	if ( is_array( $arr ) ) {
		return array_map( 'sanitize', $arr );
	}
	$ret = str_replace( "\0", '', $arr );
	$ret = unifyLineFeed( $ret );
	if ( get_magic_quotes_gpc() ) {
		$ret = stripslashes( $ret );
	}
	return $ret;
}

function unifyLineFeed( $str )
{
	return strtr( str_replace( "\x0D\x0A", "\n", $str ), "\x0D\x0A", "\n\n" );
}


if ( ! function_exists( 'file_put_contents' ) ) {

	/*
	 * PHP5 has file_put_contents()
	 */
	function file_put_contents( $filename, $contents )
	{
		clearstatcache();

		$result = FALSE;
		$mode   = is_file( $filename ) ? 'r+' : 'w';

		$fp = @fopen( $filename, $mode );
		if ( ! $fp ) {
			return $result;
		}
		flock( $fp, LOCK_EX );

		rewind( $fp );
		$num = fwrite( $fp, $contents );
		if ( is_int( $num ) ) {
			$result = ftruncate( $fp, ftell( $fp ) );
		}
		flock( $fp, LOCK_UN );
		fclose( $fp );

		clearstatcache();

		return ( $result !== FALSE ) ? $num : FALSE;
	}
}

?>
