<?php

class Net
{
	var $uri;
	var $header;
	var $raw_header;
	var $html;
	var $status;

	var $debug;

	var $proxy;

	function Net()
	{
		$this->header = array();
		$this->html   = '';
		$this->debug  = ( defined( 'DEBUG' ) && DEBUG ) ? TRUE : FALSE;
		$this->proxy  = FALSE;
		$this->status = 'OK';
	}

	function addHeaders( $arr )
	{
		// abstract method
		return FALSE;
	}

	function getContents()
	{
		// abstract method
		return FALSE;
	}

	function parseContents( $text )
	{
		// abstract method
		return FALSE;
	}

	// DEBUG
	function debugLog( $uri, $contents )
	{
		$debug_dir = '/tmp/debug/';

		if ( ! is_dir( $debug_dir ) ) {
			@mkdir( $debug_dir, 0705 );
		}
		if ( $fp = @fopen( $debug_dir . urlencode( $uri ), 'w' ) ) {
			fwrite( $fp, $contents );
			fclose( $fp );
		}
	}

	function &get( $uri, $headers = array(), $proxy = FALSE )
	{
		$net =& Net::getSubObject( $uri, (bool)$proxy );
		$net->addHeaders( $headers );
		$net->setProxy( $proxy );

		$text = $net->getContents();
		if ( $text ) {
			$net->parseContents( $text );
		}
		return $net;
	}

	function getHeader()
	{
		return $this->header;
	}

	function getHtml()
	{
		return $this->html;
	}

	function getRawHeader()
	{
		return $this->raw_header;
	}

	function getStatus()
	{
		return $this->status;
	}

	function &getSubObject( $uri, $proxy )
	{
		$dir = dirname( __FILE__ ) . '/net/';
		$arr = @parse_url( $uri );

		$host   = isset( $arr['host'] ) ? $arr['host'] : ''; 
		$scheme = ( isset( $arr['scheme'] ) && $host ) ? $arr['scheme'] : '';

		require_once( $dir . 'http.inc.php' );

		switch ( $scheme ) {
		case 'http'  :
			$net =& new Http();
			break;

		case 'ftp'   :
			require_once( $dir . 'ftp.inc.php' );
			$net =& new Ftp();
			break;

		case 'https' :
			$is_ssl  = extension_loaded( 'openssl' ) ? TRUE : @dl( 'openssl.so' );
			$is_curl = extension_loaded( 'curl' )    ? TRUE : @dl( 'curl.so' );

			if ( ( ! $is_ssl && $is_curl ) || ( $is_ssl && $is_curl && $proxy ) ) {
				require_once( $dir . 'https_proxy.inc.php' );
				$net =& new Https_proxy();
			}
			else {
				require_once( $dir . 'https.inc.php' );
				$net =& new Https();
			}
			break;

		default      :
			$net =& new Net();
			$net->setStatus( 'INVALID_URI' );
		}
		$net->setPathInfo( $arr );
		$net->setUri( $uri );

		return $net;
	}

	function isValidHost( $host )
	{
		if ( ! preg_match( "/^(?:(?:[-a-z0-9]+\.)+(?:[a-z]+)\.?|\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|localhost)$/i", $host ) ) {
			$this->status = 'INVALID_HOST';
			return FALSE;
		}
		return TRUE;
	}

	function setPathInfo( $pathinfo )
	{
		$this->pathinfo = $pathinfo;
	}

	function setProxy( $proxy )
	{
		$this->proxy = $proxy;
	}

	function setStatus( $status )
	{
		$this->status = $status;
	}

	function setUri( $uri )
	{
		$this->uri = $uri;
	}

	function talk( $host, $port, $request )
	{
		// time out : 100 seconds
		$timeout = 100;

		$fp = @fsockopen( $host, $port, $errno, $errstr, 60 );
		if ( ! $fp ) {
			$this->status = 'CONNECTION_FAILD';
			return FALSE;
		}
		socket_set_timeout( $fp, $timeout );
		fwrite( $fp, $request );

		$contents = '';
		while ( ! feof( $fp ) ) {
			$data   = @fread( $fp, 4096 );
			$status = socket_get_status( $fp );
			if ( $status['timed_out'] === FALSE ) {
				$contents .= $data;
			}
			else {
				$this->status = 'CONNECTION_TIME_OUT';
				return FALSE;
			}
		}
		fclose( $fp );
		if ( ! $contents ) {
			$this->status = 'NO_DATA';
		}
		return $contents;
	}
}

?>
