<?php

class Http extends Net
{
	var $http;
	var $gzip;
	var $addHeaders;
	var $pathinfo;

	function Http()
	{
		$this->Net();
		$this->gzip = version_compare( PHP_VERSION, '4.3.0', '>=' ) && extension_loaded( 'zlib' );
		$this->addHeaders = '';
	}

	function addHeaders( $headers = array() )
	{
		foreach ( $headers as $name => $val ) {
			switch ( $name ) {
			case 'Accept' :
			case 'User-Agent' :
			case 'If-Modified-Since' :
			case 'If-None-Match' :
			case 'Accept-Language' :
				$this->addHeaders .= $name . ': ' . trim( $val ) . "\r\n";
			}
		}
		return TRUE;
	}

	function getContents()
	{
		$result = $this->getRequest();
		if ( ! $result ) {
			return FALSE;
		}
		list( $host, $port, $request ) = $result;
		$contents = $this->talk( $host, $port, $request );

		// DEBUG
		if ( $this->debug ) {
			$this->debugLog( $this->uri, $contents );
		}
		return $contents;
	}

	function parseContents( $text )
	{
		list( $raw_header, $html ) = preg_split( "/\r?\n\r?\n/", $text, 2 );

		$header = $this->parseHeader( $raw_header );
		$this->header     = $header;
		$this->raw_header = preg_replace( "/\r?\n/", "\n", $raw_header );

		$this->setStatus( $header['status-code'] );
		if ( $this->status !== 'OK' ) {
			return FALSE;
		}
		if ( isset( $header['transfer-encoding'] ) && preg_match( "/^chunked/i", $header['transfer-encoding'] ) ) {
			$html = $this->decodeChunk( $html );
		}
		if ( isset( $header['content-encoding']  ) && preg_match( "/^gzip/i", $header['content-encoding'] ) ) {
			$html = gzinflate( substr( $html, 10 ) );
		}
		$this->html       = $html;

		return TRUE;
	}

	function checkPathInfo( $pathinfo )
	{
		if ( ! $pathinfo ) {
			$this->status = 'FATAL_ERROR';
			return FALSE;
		}
		$info['scheme'] = version_compare( PHP_VERSION, '4.3.0', '<' ) ? '' : 'tcp://';
		$info['host']   = $pathinfo['host'];
		$info['port']   = ! empty( $pathinfo['port']  ) ?  (int)$pathinfo['port']  : 80;
		$info['path']   = ! empty( $pathinfo['path']  ) ?       $pathinfo['path']  : '/';
		$info['path']  .= ! empty( $pathinfo['query'] ) ? '?' . $pathinfo['query'] : '';
		$info['user']   = ! empty( $pathinfo['user']  ) ?       $pathinfo['user']  : '';
		$info['pass']   = ! empty( $pathinfo['pass']  ) ?       $pathinfo['pass']  : '';

		if ( ! $this->isValidHost( $info['host'] ) ) {
			return FALSE;
		}
		return $info;
	}

	function getRequest()
	{
		$http = $this->checkPathInfo( $this->pathinfo );
		if ( ! $http ) {
			return FALSE;
		}
		$req = "GET " . ( $this->proxy ? $this->uri : $http['path'] ) . " HTTP/1.1\r\n"
			.  "Host: " . $http['host'] . "\r\n"
			.  $this->addHeaders
			. ( $this->gzip ? "Accept-Encoding: gzip\r\n" : '' );

		if ( $this->proxy ) {
			$req .= "Pragma: no-cache\r\n"
				.   "Cache-Control: no-cache\r\n";

			list( $proxy, $port ) = explode( ":", $this->proxy, 2 );
			$host = $http['scheme'] . $proxy;
			$port = (int)$port;
		}
		else {
			$host = $http['scheme'] . $http['host'];
			$port = (int)$http['port'];
		}

		if ( ! empty( $http['user'] ) && ! empty( $http['pass'] ) ) {
			$pass = base64_encode( $http['user'] . ':' . $http['pass'] );
			$req .= "Authorization: Basic " . $pass . "\r\n";
		}

		$req .= "Connection: close\r\n"
			.   "\r\n";

		return array( $host, $port, $req );
	}

	function parseHeader( $row_header )
	{
		$headers = preg_split( "/\r?\n/", $row_header );
		$result['status-code'] = array_shift( $headers );

		foreach ( $headers as $header ) {
			list( $key, $value ) = explode( ":", $header, 2 );
			$result[ strtolower( $key ) ] = trim( $value );
		}
		return $result;
	}

	function decodeChunk( $chunk_body )
	{
		$data = '';
		do {
			list( $size_line, $chunk_body ) = preg_split( "/\r?\n/", $chunk_body, 2 );

			if ( preg_match( '/^([0-9a-fA-F]+)/', $size_line, $matches ) ) {
				$chunk_size = hexdec( $matches[0] );
				$data      .= substr( $chunk_body, 0, $chunk_size );
				$chunk_body = ltrim( substr( $chunk_body, $chunk_size + 1 ) );
			}
		} while ( $chunk_size && $chunk_body );

		return $data;
	}

	function setStatus( $status_code )
	{
		if ( ! preg_match( '|^HTTP/1\.[01] (\d\d\d) ([\w \t]+)$|', $status_code, $code ) ) {
			$this->status = 'CANNOT_GET_STATUS_CODE';
			return FALSE;
		}
		switch ( $code[1] ) {
		case '200' :
			$this->status = 'OK';
			break;

		case '304' :
			$this->status = 'Not Modified';
			break;

		default :
			$this->status = $code[2];
			break;
		}
		return TRUE;
	}
}

?>
