<?php

class Validate
{
	/**
	 * URI regex check
	 * see http://www.ietf.org/rfc/rfc2396.txt
	 */
	function isUri( $uri )
	{
		if ( preg_match( "|[^-/?:#@&=+$,\w.!~*;'()%]|", $uri ) ) {
			return FALSE;
		}
		if ( ! preg_match( 
			  "!^(?:https?|ftp)://"						// scheme( http | https | ftp )
			. "(?:\w+:\w+@)?"							// ( user:pass )?
			. "("
			. "(?:[-_0-9a-z]+\.)+(?:[a-z]+)\.?|"		// ( domain name | IP Address | localhost )
			. "\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}|"
			. "localhost"
			. ")"
			. "(?::\d{1,5})?(?:/|$)!i",					// ( :Port )?
			$uri )
		) {
			return FALSE;
		}
		return TRUE;
	}

	/**
	 * Mail Address regex check and domain exist check
	 */
	function isMailAddr( $mail, $check_domain = FALSE )
	{
		if ( ! preg_match(
				'!^[-\\!#$%&\'*+/\w=?^_`{|}~.]+@' 
				. '([-\\!#$%&\'*+/\w=?^_`{|}~]+\.[-\\!#$%&\'*+/\w=?^_`{|}~.]+|localhost)'
				. '$!' , $mail )
		) {
			return FALSE;
		}
		if ( $check_domain ) {
			list ( , $domain ) = explode( '@', $mail );

			if ( ! checkdnsrr( $domain, 'MX' )
			  && ! checkdnsrr( $domain, 'A'  )
			  && ! checkdnsrr( $domain, 'CNAME' )
			) {
				return FALSE;
			}
		}
		return TRUE;
	}

	/**
	 * Natural Number : Integer and > 0
	 */
	function isNaturalNum( $str )
	{
		if ( ! is_bool( $str )
			&& (int)$str > 0
			&& (string)(int)$str === (string)$str
		) {
			return TRUE;
		}
		return FALSE;
	}

	/**
	 * Proxy regex check
	 */
	function isProxy( $proxy )
	{
		if ( ! preg_match( "/^(?:(?:[-\w]+\.)*([a-z]+)\.?|\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}):\d{1,5}$/i", $proxy ) ) {
			return FALSE;
		}
		return TRUE;
	}

	/**
	 * Zipcode Regex(Japan)
	 */
	function isZipCode( $code )
	{
		return preg_match( '/^\d\d\d-?\d\d\d\d$/', $code )
			? TRUE
			: FALSE;
	}

	/**
	 * Telephone Number Regex
	 */
	function isTel( $number )
	{
		return preg_match( '/^\d[\d-]+\d$/', $number )
			? TRUE
			: FALSE;
	}
}

?>
