<?php

class Command
{
	function addCategory( $category )
	{
		$cate =& Category::getInstance();
		$result = $cate->addCategory( $category );

		return array( $result ? 'INFO_ADD_CATEGORY' : 'ERROR_ADD_CATEGORY', $category );
	}

	function addItem( $uri )
	{
		if ( ! Validate::isUri( $uri ) ) {
			return array( 'ERROR_INVALID_URI', $uri, $err_code );
		}
		$item =& Command::checkSite( $uri );
		$err_code = $item->getError();
		if ( $err_code !== 'OK' ) {
			return array( 'ERROR_FAILD_ADD_ITEM', $uri, $err_code );
		}
		return array( 'INFO_ADD_ITEM', $uri, $err_code );
	}

	function alterCategory( $id, $name )
	{
		$cate =& Category::getInstance();
		$category = $cate->get( $id );
		$result   = $cate->alterCategory( $id, $name );

		return array( $result ? 'INFO_ALTER_CATEGORY' : 'ERROR_ALTER_CATEGORY', $category, $name );
	}

	function alterItem( $uri, $arr )
	{
		$result = Command::existItem( $uri );
		if ( ! $result ) {
			return array( 'ERROR_NOT_FILE', $uri );
		}
		if ( ! Validate::isUri( $arr['to_uri'] ) ) {
			return array( 'ERROR_INVALID_URI', $arr['to_uri'] );
		}
		$item =& new Item( $uri );
		$item->setProperties( $arr );
		$item->saveProperties();
		if ( $uri !== $arr['to_uri'] ) {
			$item->setProperties( array( 'uri' => $arr['to_uri'] ) );
			$item->saveProperties();
			$item->moveProperties( $arr['to_uri'] );
		}
		return array( 'INFO_CHANGE_SETTING', $arr['to_uri'] );
	}

	function &checkSite( $uri )
	{
		$conf =& Config::getInstance();

		require_once( LIB_DIR . 'baselib/net.inc.php' );

		$item =& new Item( $uri );
		$result = $item->check();
		if ( $result || $item->isFile() ) {
			$result = $item->saveLog();
			if ( ! $result ) {
				echo "Failed to write log file. URI : " . $uri . "\n";
			}
		}
		return $item;
	}

	function changePass( $user, $pass1, $pass2 )
	{
		$conf =& Config::getInstance();
		if ( ! $user || ! $pass1 || ! $pass2 ) {
			return 'ERROR_NO_USER_PASS';
		}
		if ( $pass1 !== $pass2 ) {
			return 'ERROR_NOT_VERIFY_PASS';
		}
		$conf->set( 'user', $user );
		$conf->set( 'pass', md5( $pass1 ) );
		$result = $conf->save();

		return $result ? 'INFO_CHANGE_USER_PASS' : 'ERROR_NOT_WRITE_CONF';
	}

	function checkAll( $count, $uri, $count_file )
	{
		$item =& Command::checkSite( $uri );
		$error = $item->getError();

		// increment count
		$all  = (int)file_get_contents( $count_file );
		touch( $count_file, $num = filemtime( $count_file ) + 1 );

		printf( "%4d/%4d/%4d %-13s : %s\n", $num, $count, $all, $error, $uri );
	}

	function createUniqID()
	{
		// TTL(time to live) 1800 sec.
		$ttl = 1800;

		$conf =& Config::getInstance();
		$curr = time();
		$ids = $conf->get( 'uniq_ids', array() );
		foreach ( $ids as $id => $time ) {
			if ( $time < $curr - $ttl ) {
				unset( $ids[$id] );
			}
			else {
				$uniq_id = $id;
			}
		}
		if ( count( $ids ) < 2 ) {
			if ( ! $ids || ( $curr - (int)( $ttl / 2 ) ) >= max( $ids ) ) {
				$uniq_id = md5( uniqid( TOOL_NAME ) );
				$ids[$uniq_id] = time();
			}
		}
		$conf->set( 'uniq_id', $uniq_id );
		$conf->set( 'uniq_ids', $ids );
		$conf->save();
		return $uniq_id;
	}

	function deleteItem( $uri )
	{
		$item =& new Item( $uri );
		$result = $item->delete();

		if ( ! $result ) {
			return array( 'ERROR_NOT_FILE', $uri );
		}
		return array( 'INFO_DEL_ITEM', $uri );
	}

	function delCategory( $id  )
	{
		$conf =& Config::getInstance();
		$cate =& Category::getInstance();
		$category = $cate->get( $id );
		$result   = $cate->delCategory( $id );

		// change items category from deleted category to no category.
		@include( SELF_DIR . 'misc/debug/reset_category.php' );
		reset_category( $id );

		return array( $result ? 'INFO_DEL_CATEGORY' : 'ERROR_ADD_CATEGORY', $category );
	}

	function diff( $old, $new, $tag = FALSE )
	{
		static $command = NULL;

		$conf =& Config::getInstance();

		if ( $command === NULL ) {
			$command = $conf->get( 'diff_command' );

			if ( $command ) {
				$command = Command::diffCommand( $command );
			}
			else if ( ! extension_loaded( 'xdiff' ) && @dl( 'xdiff.so' ) ) {
				$command = 'xdiff';
			}
		}
		if ( ! $tag ) {
			$old = strip_tags( $old );
			$new = strip_tags( $new );
		}
		if ( $command === 'xdiff' ) {
			$diff = xdiff_string_diff( $old, $new );
		}
		else if ( $command ) {
			$tmp_old = tempnam( TEMP_DIR, "PWA_OLD" );
			file_put_contents( $tmp_old, $old );

			$tmp_new = tempnam( TEMP_DIR, "PWA_NEW" );
			file_put_contents( $tmp_new, $new );

			$diff = shell_exec( $command . ' ' . $tmp_old . ' ' . $tmp_new );

			@unlink( $tmp_old );
			@unlink( $tmp_new );
		}
		else {
			require_once( LIB_DIR . 'baselib/diff.inc.php' );
			$diff =	diff_string( $old, $new );
		}
		return trim(
			preg_replace(
				array( "/&nbsp;/", "/^(?:[^>+].*$|[>+]\s*)/m", "/\n+/", "/[\t ]+/" ),
				array( " ",        "",                         "\n",    " "        ),
				$diff
			)
		);
	}

	function diffCommand( $diff )
	{
		if ( ! is_executable( $diff ) ) {
			$diff = 'diff';
		}
		$result = @shell_exec( $diff . ' --version' );
		if ( ! preg_match( '/^diff/', $result ) ) {
			return FALSE;
		}
		return $diff;
	}

	function diffFile( $old_file, $new_file, $tag = FALSE )
	{
		if ( ! is_readable( $old_file ) || ! is_readable( $new_file ) ) {
			return '';
		}
		return Command::diff(
			file_get_contents( $old_file ),
			file_get_contents( $new_file ),
			$tag
		);
	}

	function existItem( $uri )
	{
		$item =& new Item( $uri );
		if ( ! $uri || ! $item->isFile() ) {
			return FALSE;
		}
		return $uri;
	}

	function getMessage( $id, $option )
	{
		$conf   =& Config::getInstance();
		$lang    = $conf->get( 'message_lang' );
		$file    = VIEW_DIR . $lang . '/messages.php';

		$option  = array_pad( $option, 10, '' );
		$message = include( $file );
		if ( isset( $message[$id] ) ) {
			return $message[$id];
		}
		return '';
	}

	function isCsrf( $uniq_id )
	{
		$ttl   = 1800;
		$conf =& Config::getInstance();
		$ids   = $conf->get( 'uniq_ids' );
		if ( isset( $ids[$uniq_id] ) && $ids[$uniq_id] > time() - $ttl ) {
			return FALSE;
		}
		return TRUE;
	}

	function phpCli( $php )
	{
		if ( ! is_executable( $php ) ) {
			$php = 'php';
		}
		$result = @shell_exec( $php . ' --version' );
		if ( ! preg_match( '/^PHP ([4-9]\.\d\.\d+(?:(?:dev|alpha|beta|RC|pl)\d*)?) \((cli|cgi(?:-fcgi)?)\)/', $result, $m ) ) {
			return FALSE;
		}
		if ( version_compare( $m[1], '4.3.0', '<' ) ) {
			return FALSE;
		}
		if ( $m[2] !== 'cli' ) {
			$php .= ' -q';
		}
		return $php;
	}

	function redirect( $query = '', $mid = '', $opt = array() )
	{
		$serv =& Request::getInstance( '_SERVER' );
		$self = $serv->get( 'PHP_SELF' );
		$host = $serv->get( 'HTTP_HOST' );
		$port = $serv->get( 'SERVER_PORT' );

		if ( $serv->get( 'HTTPS' ) === 'on' ) {
			$scheme = 'https://';
			$port   = ( $port !== 443 ) ? ':' . (int)$port : '';
		}
		else {
			$scheme = 'http://';
			$port   = ( $port !== 80  ) ? ':' . (int)$port : '';
		}
		if ( $mid ) {
			$conf   =& Config::getInstance();
			$message = Command::getMessage( $mid, $opt );
			$id      = uniqid( 'm' );
			$query  .= ( $query ? '&' : '?' ) . 'mid=' . $id;
			$conf->set( 'message', $message );
			$conf->set( 'mid',     $id );
			$conf->save();
		}
		header( 'Location: ' . $scheme . $host . $port . $self . $query );
		exit;
	}

	function setConfig( $array )
	{
		$conf =& Config::getInstance();
		foreach ( $array as $key => $val ) {
			$conf->set( $key, $val );
		}
		$result = $conf->save();
		return $result ? 'INFO_CHANGE_SETTING' : 'ERROR_NOT_WRITE_CONF';
	}

	function setProxy( $use_proxy, $proxy )
	{
		$conf =& Config::getInstance();
		if ( ! $proxy ) {
			$use_proxy = FALSE;
		}
		else if ( ! Validate::isProxy( $proxy ) ) {
			$proxy     = '';
			$use_proxy = FALSE;
		}
		$conf->set( 'use-proxy', (bool)$use_proxy );
		$conf->set( 'proxy',     $proxy );
		$result = $conf->save();

		return $result
			? array( 'INFO_CHANGE_PROXY', $use_proxy, $proxy )
			: array( 'ERROR_NOT_WRITE_CONF', '', '' );
	}

	function updateItem( $uri )
	{
		$result = Command::existItem( $uri );
		if ( ! $result ) {
			return array( 'ERROR_NOT_FILE', $uri );
		}
		$item =& Command::checkSite( $uri );
		$code  = $item->getError();
		if ( $code === 'OK' ) {
			return array( 'INFO_UPDATE_ITEM', $uri, $code );
		}
		elseif ( $code === 'Not Modified' ) {
			return array( 'INFO_NOT_MODIFIED', $uri, $code );
		}
		return array( 'ERROR_UPDATE_ITEM', $uri, $code );
	}

	function upCategory( $id )
	{
		$cate =& Category::getInstance();
		$category = $cate->get( $id );
		$result   = $cate->upCategory( $id );

		return array( $result ? 'INFO_UP_CATEGORY' : 'ERROR_MOVE_CATEGORY', $category );
	}

	function downCategory( $id )
	{
		$cate =& Category::getInstance();
		$category = $cate->get( $id );
		$result   = $cate->downCategory( $id );

		return array( $result ? 'INFO_DOWN_CATEGORY' : 'ERROR_MOVE_CATEGORY', $category );
	}
}

?>
