<?php

class Filters
{
	var $filter_file;
	var $patterns;

	function Filters( $filter_file = '' )
	{
		$this->patterns = array();
		if ( $filter_file ) {
			$this->filter_file = $filter_file;
			$this->setFilterPatterns( $filter_file );
		}
	}

	function &getInstance( $filter_file = '' )
	{
		static $instance;

		if ( $instance === NULL ) {
			$instance = new Filters( $filter_file );
		}
		return $instance;
	}

	function add( $uri, $pattern )
	{
		$filters = $this->getRawFilters();
		if ( isset( $filters[$uri] ) ) {
			array_push( $filters[$uri], $pattern );
		}
		else {
			$filters[$uri] = array( $pattern );
		}
		$this->save( $filters );
	}

	function del( $uri, $pattern )
	{
		$filters = $this->getRawFilters();
		if ( ! isset( $filters[$uri] ) ) {
			return FALSE;
		}
		foreach ( $filters[$uri] as $num => $pat ) {
			if ( $pattern === $pat ) {
				unset( $filters[$uri][$num] );
				if ( count( $filters[$uri] ) ) {
					sort( $filters[$uri] );
				}
				else {
					unset( $filters[$uri] );
				}
				break;
			}
		}
		$this->save( $filters );
	}

	function exec( $html, $uri )
	{
		$html = unifyLineFeed( $html );

		foreach ( $this->patterns as $patterns ) {
			if ( preg_match( $patterns['domain'], $uri ) ) {
				$html = preg_replace( $patterns['pattern'], '', $html );
			}
		}
		return $html;
	}

	function getFilters()
	{
		return $this->patterns;
	}

	function getRawFilters()
	{
		$filters = loadConfig( $this->filter_file );
		if ( $filters === FALSE || ! is_array( $filters ) ) {
			$filters = array();
		}
		return $filters;
	}

	function quote( $arg )
	{
		if ( is_array( $arg ) ) {
			return array_map( array( $this, 'quote' ), $arg );
		}
		return '/' . str_replace( "/", "\\/", $arg ) . '/is';
	}

	function save( $filters )
	{
		return saveConfig( $this->filter_file, $filters );
	}

	function setFilterPatterns( $filter_file )
	{
		$patterns = loadConfig( $filter_file );
		if ( $patterns === FALSE || ! is_array( $patterns ) ) {
			$patterns = array();
		}
		$this->patterns = array();
		foreach ( $patterns as $domain => $pattern ) {
			$this->patterns[] = array(
				'domain'  => '/' . preg_quote( $domain, '/' ) . '/i',
				'pattern' => $this->quote( $pattern )
			);
		}
		return TRUE;
	}

	function test( $uri, $from )
	{
		$filters = $this->getRawFilters();
		if ( isset( $filters[$uri] ) ) {
			$patterns = $this->quote( $filters[$uri] );
		}
		$result = preg_replace( $patterns, '%%begin_filter_delete_mark%%\\0%%end_filter_delete_mark%%', $from );
		$result = preg_replace( '/%%begin_filter_delete_mark%%(.*?)%%end_filter_delete_mark%%/is', '<strong>$1</strong>', htmlspecialchars( $result ) );
		$result = preg_replace( '/%%begin_filter_delete_mark%%|%%end_filter_delete_mark%%/', '', $result );
		return $result;
	}
}

?>
