<?php

class Item
{
	var $check_uri;
	var $html;
	var $is_file;
	var $log_file;
	var $period;
	var $profile;
	var $writer;

	function Item( $uri )
	{
		$conf =& Config::getInstance();

		$encode_uri = urlencode( $uri );

		$this->writer    =& Data::getWriter( SAVE_HANDLE, $encode_uri );
		$this->check_uri = $uri;
		$this->html      = '';
		$this->is_file   = TRUE;
		$this->period    = $conf->get( 'log_period', 4 );
		$this->log_file  = LOG_DIR .  $encode_uri;
		$this->profile   = $this->writer->load();

		if ( ! is_array( $this->profile ) ) {
			$this->profile = array();
			$this->is_file = FALSE;
		}
		$this->profile['uri']      = $uri;
		$this->profile['error']    = '';
		$this->profile['count']    = ! empty( $this->profile['count']    ) ? (int)$this->profile['count'] : 0;
		$this->profile['encoding'] = ! empty( $this->profile['encoding'] ) ? $this->profile['encoding']   : FALSE;
	}

	function check()
	{
		$profile =& $this->profile;

		$conf =& Config::getInstance();
		$proxy = $conf->get( 'use-proxy' ) ? $conf->get( 'proxy' ) : FALSE;

		$add_headers = array(
			'Accept'          => '*/*',
			'Accept-Language' => $conf->get( 'net_accept_lang', 'ja' ),
			'User-Agent'      => $conf->get( 'net_user_agent',  TOOL_NAME . ' ' . TOOL_VERSION )
		);
		// when "ETag" exists, add 'If-Modified-Since' header and 'If-None-Match' header.
		if ( isset( $profile['header'] ) && preg_match( '/^ETag:\s*(.*)$/im', $profile['header'], $m ) ) {
			$add_headers['If-None-Match']     = $m[1];
			$add_headers['If-Modified-Since'] = gmdate( 'D, d M Y H:i:s T', $profile['last_modified'] );
		}
		$net =& Net::get( $this->check_uri, $add_headers, $proxy );

		$status = $net->getStatus();
		$profile['error']  = $status;
		$profile['status'] = $profile['error'];

		// when HTTP Header exists, update $profile['header']
		$row_header = $net->getRawHeader();
		if ( $row_header ) {
			$profile['header'] = $row_header;
		}
		if ( $status !== 'OK' ) {
			return FALSE;
		}
		$html   = $net->getHtml();
		$header = $net->getHeader();

		// detect encoding from http header or meta tag
		$enc =& Encodings::getInstance();
		$charset = '';
		if ( $profile['encoding'] ) {
			$charset = $profile['encoding'];
		}
		else if ( ! empty( $header['content-type'] ) ) {
			$charset = $enc->getHtmlCharset( $header['content-type'] );
		}
		if ( ! $charset ) {
			$charset = $enc->getHtmlCharset( $html );
		}
		$profile['charset'] = $enc->detectEncoding( $html, $charset );
		$profile['size']    = strlen( $html );

		$html = $enc->convert( $html );

		// execute html filter
		$filters =& Filters::getInstance();
		$html    = $filters->exec( $html, $this->check_uri );

		if ( ! isset( $profile['title'] ) ) {
			$profile['title'] = $this->getTitle( $html );
		}
		if ( ! isset( $profile['category'] ) ) {
			$profile['category'] = '';
		}
		if ( ! empty( $header['content-length'] ) ) {
			$profile['size'] = (int)$header['content-length'];
		}
		$diff = $this->getDiff( $this->log_file, $html );
		if ( $diff !== FALSE ) {
			$profile['diff'] = $diff;
		}

		$last_modified = isset( $profile['last_modified'] ) ? (int)$profile['last_modified'] : -1;
		$last_modified = $this->getLastModified( $header, $last_modified, (bool)$diff );
		$profile['last_modified'] = $last_modified;

		$profile['last_check'] = time();
		if ( $profile['last_modified'] > $profile['last_check'] ) {
			$profile['last_modified'] = $profile['last_check'];
		}
		if ( empty( $profile['last_visit'] ) ) {
			$profile['last_visit'] = 0;
		}
		$this->html = $html;

		return TRUE;
	}

	function delete()
	{
		$result = $this->writer->delete();
		if ( ! $result ) {
			return FALSE;
		}
		// delete log file
		@unlink( $this->log_file );
		$i = 0;
		while ( is_writable( $this->log_file . '.' . ++$i ) ) {
			@unlink( $this->log_file . '.' . $i );
		}
		return TRUE;
	}

	function getDiff( $log_file, $html )
	{
		$diff = FALSE;
		if ( is_readable( $log_file ) ) {
			$log = file_get_contents( $this->log_file );
			if ( $log !== $html ) {
				$diff = Command::diff( $log, $html );
			}
		}
		return $diff;
	}

	function getError()
	{
		return ( ! empty( $this->profile['error'] ) )
			? $this->profile['error']
			: FALSE;
	}

	function getLastModified( $header, $last_modified, $diff_flag )
	{
		$last = -1;
		// check http header 'Last-Modified'
		if ( isset( $header['last-modified'] ) ) {
			$last = strtotime( $header['last-modified'] );
			if ( ! $diff_flag ) {
				if ( $last_modified > 0 ) {
					$last = $last_modified;
				}
				else {
					$last = is_readable( $this->log_file )
						? filemtime( $this->log_file )
						: $last;
				}
			}
		}
		if ( $last < 0 ) {
			if ( $diff_flag ) {
				$last = isset( $header['date'] )
					? strtotime( $header['date'] )
					: time();
			}
			else {
				$last = ( $last_modified >= 0 )
					? $last_modified
					: is_file( $this->log_file ) && is_readable( $this->log_file ) 
						? filemtime( $this->log_file )
						: time();
			}
		}
		return $last;
	}

	function getProperties()
	{
		return $this->profile;
	}

	function getTitle( $html )
	{
		return preg_match( '/<title.*?>(.+?)<\/title\s*>/is', $html, $arr ) 
			? $arr[1]
			: $this->check_uri;
	}

	function redirect( $uri )
	{
		$item =& new Item( $uri );
		$item->goUri();
	}

	function goUri()
	{
		if ( ! $this->isFile() ) {
			return FALSE;
		}
		$conf =& Config::getInstance();
		if ( $conf->isAuth() ) {
			$this->profile['last_visit'] = time();
		}
		++$this->profile['count'];

		$this->saveProperties();

		$open_uri = ! empty( $this->profile['open_uri'] )
			? $this->profile['open_uri']
			: $this->check_uri;

		header( "Location: " . strip_tags( $open_uri ) );
		exit;
	}

	function isFile()
	{
		return $this->is_file;
	}

	function moveProperties( $to_uri )
	{
		$encode_uri = urlencode( $to_uri );

		// rename item
		$result = $this->writer->move( $encode_uri );
		if ( ! $result ) {
			return FALSE;
		}
		// rename logs
		if ( ! is_writable( $this->log_file ) ) {
			return FALSE;
		}
		$to_logfile = LOG_DIR . $encode_uri;
		rename( $this->log_file, $to_logfile );
		$i = 0;
		while ( is_writable( $this->log_file . '.' . ++$i ) ) {
			rename( $this->log_file . '.' . $i, $to_logfile . '.' . $i );
		}
		return TRUE;
	}

	function rotateLogs( $filename, $period )
	{
		while ( --$period >= 0 ) {
			$logfile = $filename . ( $period ?  '.' . $period : '' );
			$oldfile = $filename . '.' . ( $period + 1 );
			if ( is_writable( $logfile ) ) {
				@unlink( $oldfile );
				rename( $logfile, $oldfile );
			}
		}
		clearstatcache();
	}

	function saveLog()
	{
		$this->saveProperties();

		// check failed : do not make log
		if ( $this->getError() !== 'OK' ) {
			return TRUE;
		}

		if ( is_readable( $this->log_file ) ) {
			if ( file_get_contents( $this->log_file ) === $this->html ) {
				$result = TRUE;
			}
			else {
				$this->rotateLogs( $this->log_file, $this->period );
				$result = file_put_contents( $this->log_file, $this->html );
			}
		}
		else {
			$result = file_put_contents( $this->log_file, $this->html );
		}
		return $result;
	}

	function saveProperties()
	{
		return $this->writer->save( $this->profile );
	}

	function setProperties( $arr )
	{
		foreach ( $arr as $key => $val ) {
			$this->profile[$key] = $val;
		}
	}
}

?>
