<?php
// for CGI
//   #!/usr/local/bin/php

// timer start
$ts = microtime();

// read modules
require( 'init.inc.php' );
require( LIB_DIR . 'command.inc.php' );
require( LIB_DIR . 'filters.inc.php' );

Filters::getInstance( $filter_file );

// disable time limit
@set_time_limit( 0 );

// create configuration object
$conf =& Config::getInstance();
$serv =& Request::getInstance( '_SERVER' );

// prevent double starting
if ( is_file( $count_file ) && ( filectime( $count_file ) + 1000 > time() ) ) {
	exit( "This script has just execute. exit.\n" );
}

$uris = array();
$sapi = $serv->get( 'DOCUMENT_ROOT' ) ? PHP_SAPI : 'cli';

// HTML for Apache DSO or CGI version 
if ( $sapi !== 'cli' ) {
	// require HTTP authentication with PHP
	if ( ! $conf->isAuth() ) {
		include( VIEW_DIR . 'error_auth.tpl.php' );
		exit;
	}
	// POST only and not CSRF
	if ( ! isset( $_POST['update'] ) || ! isset( $_POST['uniq_id'] ) || Command::isCsrf( $_POST['uniq_id'] ) ) {
		Command::redirect();
		exit;
	}

	// enable implicit flush.
	ob_implicit_flush();

	// ignore user abort
	ignore_user_abort( 1 );

	// clear output buffers
	while ( @ob_end_flush() );

	echo "<html><head><title>Check...</title></head><body>\n<pre>\n";
}
else {
	// for PHP CLI
	$argc = $serv->get( 'argc' ) ? (int)$serv->get( 'argc' ) : ( isset( $argc ) ? (int)$argc : 0       );
	$argv = $serv->get( 'argv' ) ?      $serv->get( 'argv' ) : ( isset( $argv ) ?      $argv : array() );

	// get items from commandline args
	if ( $argc > 1 ) {
		for ( $i = 1; $i < $argc; ++$i ) {
			if ( Validate::isUri( $argv[$i] ) ) {
				$uris[] = $argv[$i];
			}
		}
	}
}
echo "Start Time: " . date( "Y/m/d H:i:s" ) . "\n";
echo $conf->get( 'use-proxy' ) ? "Using Proxy : " . $conf->get( 'proxy' ) . "\n\n" : "\n";

if ( ! $uris ) {
	// get all items
	$reader =& Data::getReader( SAVE_HANDLE, $dummy = '' );
	$uris = $reader->getList();
	if ( ! $uris ) {
		exit( "Could not open data file or find item.\n" );
	}
}

// initialize count and count file
@unlink( $count_file );
file_put_contents( $count_file, count( $uris ) );
touch( $count_file, 0 );

$count   = 0;
$pids    = array();
$process = 6;

if ( extension_loaded( 'pcntl' ) && ( $sapi === 'cli' || $sapi === 'cgi' ) ) {

	foreach ( $uris as $uri ) {
		++$count;
		$pid = pcntl_fork();
		if ( $pid === -1 ) {
			exit( "could not fork.\n" );
		}
		else if ( $pid ) {
			// parent process
			$pids[ $pid ] = TRUE;
			if ( next( $uris ) === FALSE ) {
				$process = 1;
			}
			while ( count( $pids ) >= $process ) {
				unset( $pids[ pcntl_waitpid( -1, $status, WUNTRACED ) ] );
			}
		}
		else {
			// child process
			Command::checkAll( $count, $uri, $count_file );
			exit;
		}
	}
}
else if ( $php = Command::phpCli( $conf->get( 'php_command' ) ) ) {

	foreach ( $uris as $uri ) {
		// execute php cli by popen()
		$c = $php . ' ' . SELF_DIR . 'check.php ' . ++$count . ' ' . $uri;
		$p = popen( $c, 'r' );
		if ( $p === FALSE ) {
			echo "could not execute popen. command : " . $c . "\n";
			continue;
		}
		stream_set_blocking( $p, FALSE );
		$pids[] = $p;
		if ( next( $uris ) === FALSE ) {
			$process = 1;
		}
		while ( count( $pids ) >= $process ) {
			foreach ( $pids as $n => $p ) {
				echo fread( $p, 1024 );
				if ( feof( $p ) ) {
					pclose( $p );
					unset( $pids[ $n ] );
				}
			}
			usleep( 10000 );
		}
	}
}
else {
	foreach ( $uris as $uri ) {
		Command::checkAll( ++$count, $uri, $count_file );
	}
}
// delete count file
@unlink( $count_file );

// update configuration file
$conf->set( 'last-modified', time() );
$result = $conf->save();
if ( ! $result ) {
	echo "\n--\n";
	echo "Cannot write configuration file. file : " . $conf->getConfigFile() . "\n";
	echo "Please check file permission.\n";
}

// end time
echo "\n--\n";
echo "End Time: " . date( "Y/m/d H:i:s" ) . "\n";

// timer end, print message
diffTime( $ts );
printf( "END. %f Seconds\n", diffTime() );

if ( $sapi !== 'cli' ) {
	echo "</pre>\n<div><a href=\"./\">Go to Result Page</a></div></body></html>";
}

$vacuum_file = SELF_DIR . 'misc/' . SAVE_HANDLE . '/vacuum.php';
if ( is_file( $vacuum_file ) ) {
	include( $vacuum_file );
}

?>
